<?php

/*
 * This file is part of Resume.PHP.
 *
 * (copyleft) R. Eric Wheeler <sikofitt@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

/**
 * Created by PhpStorm.
 * User: eric
 * Date: 7/3/16
 * Time: 7:37 PM
 */

namespace Sikofitt\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Webmozart\Json\JsonDecoder;
use Webmozart\Json\JsonValidator;

/**
 * Class TestCommand
 * @package Sikofitt\Command
 */
class SchemaValidationCommand extends Command
{
    private $file;
    private $schema;

    /**
     * Configures the command.
     *
     */
    public function configure()
    {
        $this->setName('resume:schema:validate')
            ->setDescription('Validates resume.json schema')
            ->setProcessTitle('Validating Schema');
    }

    /**
     * Unset parameters that we need to possibly get.
     *
     * @param InputInterface $input
     * @param OutputInterface $output
     */
    public function interact(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $this->file = $io->ask('Path to file? ');
        $this->schema = $io->ask('Path to schema? ');
    }

    /**
     * Command execute logic.
     *
     * @param InputInterface $input
     * @param OutputInterface $output
     *
     * @return void
     */
    public function execute(InputInterface $input, OutputInterface $output)
    {
        $validator = new JsonValidator();
        $decoder = new JsonDecoder();
        $io = new SymfonyStyle($input, $output);
        $dataDir = dirname(dirname(dirname(__DIR__))) . '/data/';

        try {
            $decoded = $decoder->decodeFile($dataDir . $this->file);
            $errors = $validator->validate($decoded, $dataDir . $this->schema);
        } catch (\Exception $e) {
            $class = new \ReflectionClass($e);
            $io->block([$class->getName(), $e->getMessage()], 'Exception', 'error');
            exit(255);
        }
        if (isset($errors) && !empty($errors)) {
            $io->block($errors, 'Error', 'error');
        } else {
            $io->block([$this->file, 'There were no errors reported.'], 'Valid Schema', 'fg=black;bg=green');
        }
    }
}
