<?php

/*
 * This file is part of Resume.PHP.
 *
 * (copyleft) R. Eric Wheeler <sikofitt@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Sikofitt\Controller;

use Sikofitt\{
    Resume\Pdf,
    Form\Type\ContactType,
    Resume\ResumeParser
};
use Silex\{
    Api\ControllerProviderInterface,
    Application,
    ControllerCollection
};
use Symfony\Component\HttpFoundation\ {
    Request,
    Response,
    ResponseHeaderBag,
    StreamedResponse
};

/**
 * Class ResumeControllerProvider
 *
 * @package Sikofitt\Controller
 */
class ResumeControllerProvider implements ControllerProviderInterface
{

    /**
     * @var ResumeParser
     */
    private $resume;

    /**
     * @param Application $app
     *
     * @return ControllerCollection
     */
    public function connect(Application $app)
    {
        $this->resume = new ResumeParser($app);

        $controllers = $app['controllers_factory'];

        $controllers->get('/', function (Request $request) use ($app) {

            $contactForm = $app['form.factory']->create(ContactType::class);

            return $app['twig']->render('index.html.twig', [
                'renderPdf'    => false,
                'baseUrl'      => $request->getSchemeAndHttpHost(),
                'basics'       => $this->resume->getBasics(),
                'work'         => $this->resume->getWork(),
                'volunteer'    => $this->resume->getVolunteer(),
                'education'    => $this->resume->getEducation(),
                'awards'       => $this->resume->getAwards(),
                'publications' => $this->resume->getPublications(),
                'skills'       => $this->resume->getSkills(),
                'languages'    => $this->resume->getLanguages(),
                'interests'    => $this->resume->getInterests(),
                'references'   => $this->resume->getReferences(),
                'contact_form' => $contactForm->createView(),
            ]);
        });

        $controllers->get('/pdf', function (Request $request) use ($app) {

            $pdf = new Pdf($app, $request);

            $renderedPdf = $pdf->render();

            $filename = str_replace('@', '-AT-', $this->resume->getBasics()->email);

            $response = new StreamedResponse();
            $response->setCallback(function () use ($renderedPdf) {
                print $renderedPdf;
            });

            $response->setStatusCode(Response::HTTP_OK);
            $disposition = $response->headers->makeDisposition(
                ResponseHeaderBag::DISPOSITION_ATTACHMENT,
                $filename . '.pdf'
            );
            $response->headers->set('Content-Disposition', $disposition);
            $response->send();
        });



        return $controllers;
    }
}
